@@ -182,7 +182,7 @@ FCITX_CONFIGURATION(
         this, "RomajiHalfNumber", _("Use half-width characters for numbers"),
         true};
     fcitx::OptionWithAnnotation<InputMode, InputModeI18NAnnotation> inputMode{
-        this, "InputMode", _("Input mode"), InputMode::HIRAGANA};
+        this, "InputMode", _("Initial Input mode"), InputMode::HIRAGANA};
     fcitx::OptionWithAnnotation<TypingMethod, TypingMethodI18NAnnotation>
         typingMethod{this, "TypingMethod", _("Typing method"),
                      TypingMethod::ROMAJI};
@@ -457,13 +457,13 @@ FCITX_CONFIGURATION(
         this,
         "MoveCaretForwardKey",
         _("Move caret forward"),
-        {fcitx::Key("Control+F"), fcitx::Key(FcitxKey_Left)},
+        {fcitx::Key("Control+F"), fcitx::Key(FcitxKey_Right)},
         fcitx::KeyListConstrain(fcitx::KeyConstrainFlag::AllowModifierLess)};
     fcitx::KeyListOption hk_MOVE_CARET_BACKWARD{
         this,
         "MoveCaretBackwardKey",
         _("Move caret backward"),
-        {fcitx::Key("Control+B"), fcitx::Key(FcitxKey_Right)},
+        {fcitx::Key("Control+B"), fcitx::Key(FcitxKey_Left)},
         fcitx::KeyListConstrain(fcitx::KeyConstrainFlag::AllowModifierLess)};
     fcitx::KeyListOption hk_SELECT_FIRST_SEGMENT{
         this,
