@@ -8,21 +8,23 @@
 #include "engine.h"
 #include "action.h"
 #include "state.h"
-#include <errno.h>
+#include "style_file.h"
+#include <cerrno>
+#include <cstdio>
 #include <fcitx-config/iniparser.h>
 #include <fcitx-utils/log.h>
 #include <fcitx-utils/standardpath.h>
+#include <fcitx-utils/stringutils.h>
 #include <fcitx/addonfactory.h>
 #include <fcitx/addonmanager.h>
 #include <fcitx/inputcontext.h>
 #include <fcitx/inputcontextmanager.h>
 #include <fcitx/userinterfacemanager.h>
-#include <stdio.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
 
-FCITX_DEFINE_LOG_CATEGORY(anthy_logcategory, "anthy");
+FCITX_DEFINE_LOG_CATEGORY(anthy_logcategory, "anthy")
 
 class AnthyFactory : public fcitx::AddonFactory {
     fcitx::AddonInstance *create(fcitx::AddonManager *manager) override {
@@ -129,7 +131,7 @@ struct AnthyModeTraits<InputMode>
     : AnthyStatusHelper<InputMode, &input_mode_status> {
     static auto mode(AnthyState *state) { return state->inputMode(); }
     static void setMode(AnthyState *state, InputMode mode) {
-        return state->setInputMode(mode);
+        return state->setInputMode(mode, /*propagate=*/true);
     }
     static const char *icon(InputMode mode) {
         if (auto *s = status(mode)) {
@@ -151,7 +153,7 @@ struct AnthyModeTraits<TypingMethod>
     : AnthyStatusHelper<TypingMethod, &typing_method_status> {
     static auto mode(AnthyState *state) { return state->typingMethod(); }
     static void setMode(AnthyState *state, TypingMethod mode) {
-        return state->setTypingMethod(mode);
+        return state->engine()->setTypingMethod(mode);
     }
     static auto label(TypingMethod mode) {
         if (auto *s = status(mode)) {
@@ -164,9 +166,11 @@ struct AnthyModeTraits<TypingMethod>
 template <>
 struct AnthyModeTraits<ConversionMode>
     : AnthyStatusHelper<ConversionMode, &conversion_mode_status> {
-    static auto mode(AnthyState *state) { return state->conversionMode(); }
+    static auto mode(AnthyState *state) {
+        return state->engine()->conversionMode();
+    }
     static void setMode(AnthyState *state, ConversionMode mode) {
-        return state->setConversionMode(mode);
+        return state->engine()->setConversionMode(mode);
     }
     static std::string label(ConversionMode mode) {
         if (auto *s = status(mode)) {
@@ -180,25 +184,29 @@ struct AnthyModeTraits<ConversionMode>
 template <>
 struct AnthyModeTraits<PeriodCommaStyle>
     : AnthyStatusHelper<PeriodCommaStyle, &period_style_status> {
-    static auto mode(AnthyState *state) { return state->periodCommaStyle(); }
+    static auto mode(AnthyState *state) {
+        return state->engine()->periodCommaStyle();
+    }
     static void setMode(AnthyState *state, PeriodCommaStyle mode) {
-        return state->setPeriodCommaStyle(mode);
+        return state->engine()->setPeriodCommaStyle(mode);
     }
 };
 
 template <>
 struct AnthyModeTraits<SymbolStyle>
     : AnthyStatusHelper<SymbolStyle, &symbol_style_status> {
-    static auto mode(AnthyState *state) { return state->symbolStyle(); }
+    static auto mode(AnthyState *state) {
+        return state->engine()->symbolStyle();
+    }
     static void setMode(AnthyState *state, SymbolStyle mode) {
-        return state->setSymbolStyle(mode);
+        return state->engine()->setSymbolStyle(mode);
     }
 };
 
 template <typename ModeType>
 class AnthyAction : public fcitx::Action {
 public:
-    AnthyAction(AnthyEngine *engine) : engine_(engine) {}
+    explicit AnthyAction(AnthyEngine *engine) : engine_(engine) {}
     std::string shortText(fcitx::InputContext *ic) const override {
         auto state = engine_->state(ic);
         auto mode = AnthyModeTraits<ModeType>::mode(state);
@@ -262,6 +270,29 @@ AnthyEngine::AnthyEngine(fcitx::Instance *instance)
     : instance_(instance), factory_([this](fcitx::InputContext &ic) {
           return new AnthyState(&ic, this, instance_);
       }) {
+    anthy_set_logger(
+        [](int level, const char *msg) {
+            FCITX_UNUSED(level);
+            FCITX_ANTHY_INFO() << "Anthy: " << msg;
+        },
+        0);
+#ifdef __ANDROID__
+    const auto &sp = fcitx::StandardPath::global();
+    std::string anthy_conf =
+        sp.locate(fcitx::StandardPath::Type::Data, "anthy/anthy-unicode.conf");
+    std::string anthy_prefix = fcitx::fs::dirName(anthy_conf);
+    // "CONFFILE" must be overridden first to change main config file path
+    anthy_conf_override("CONFFILE", anthy_conf.c_str());
+    // "prefix" must be set before using "${prefix}" in other values
+    anthy_conf_override("prefix", anthy_prefix.c_str());
+    anthy_conf_override("ANTHYDIR", "${prefix}/");
+    anthy_conf_override("DIC_FILE", "${prefix}/anthy.dic");
+    // save anthy data (primarily input history) to
+    // /sdcard/Android/data/<pkg>/files/data/anthy
+    anthy_conf_override(
+        "XDG_CONFIG_HOME",
+        sp.userDirectory(fcitx::StandardPath::Type::Data).c_str());
+#endif
     if (anthy_init()) {
         throw std::runtime_error("Failed to init anthy library.");
     }
@@ -300,49 +331,49 @@ AnthyEngine::AnthyEngine(fcitx::Instance *instance)
     type##Menu_.addAction(subModeActions_.back().get());
 
     _ADD_ACTION(InputMode::HIRAGANA, InputMode, inputMode,
-                "anthy-input-mode-hiragana");
+                "anthy-input-mode-hiragana")
     _ADD_ACTION(InputMode::KATAKANA, InputMode, inputMode,
-                "anthy-input-mode-katakana");
+                "anthy-input-mode-katakana")
     _ADD_ACTION(InputMode::HALF_KATAKANA, InputMode, inputMode,
-                "anthy-input-mode-half-katakana");
+                "anthy-input-mode-half-katakana")
     _ADD_ACTION(InputMode::LATIN, InputMode, inputMode,
-                "anthy-input-mode-latin");
+                "anthy-input-mode-latin")
     _ADD_ACTION(InputMode::WIDE_LATIN, InputMode, inputMode,
-                "anthy-input-mode-wide-latin");
+                "anthy-input-mode-wide-latin")
 
     _ADD_ACTION(TypingMethod::ROMAJI, TypingMethod, typingMethod,
-                "anthy-typing-method-romaji");
+                "anthy-typing-method-romaji")
     _ADD_ACTION(TypingMethod::KANA, TypingMethod, typingMethod,
-                "anthy-typing-method-kana");
+                "anthy-typing-method-kana")
     _ADD_ACTION(TypingMethod::NICOLA, TypingMethod, typingMethod,
-                "anthy-typing-method-nicola");
+                "anthy-typing-method-nicola")
 
     _ADD_ACTION(ConversionMode::MULTI_SEGMENT, ConversionMode, conversionMode,
-                "anthy-conversion-mode-multi");
+                "anthy-conversion-mode-multi")
     _ADD_ACTION(ConversionMode::SINGLE_SEGMENT, ConversionMode, conversionMode,
-                "anthy-conversion-mode-single");
+                "anthy-conversion-mode-single")
     _ADD_ACTION(ConversionMode::MULTI_SEGMENT_IMMEDIATE, ConversionMode,
-                conversionMode, "anthy-conversion-mode-multi-imm");
+                conversionMode, "anthy-conversion-mode-multi-imm")
     _ADD_ACTION(ConversionMode::SINGLE_SEGMENT_IMMEDIATE, ConversionMode,
-                conversionMode, "anthy-conversion-mode-single-imm");
+                conversionMode, "anthy-conversion-mode-single-imm")
 
     _ADD_ACTION(PeriodCommaStyle::WIDELATIN, PeriodCommaStyle, periodStyle,
-                "anthy-period-widelatin");
+                "anthy-period-widelatin")
     _ADD_ACTION(PeriodCommaStyle::LATIN, PeriodCommaStyle, periodStyle,
-                "anthy-period-latin");
+                "anthy-period-latin")
     _ADD_ACTION(PeriodCommaStyle::JAPANESE, PeriodCommaStyle, periodStyle,
-                "anthy-period-japanese");
+                "anthy-period-japanese")
     _ADD_ACTION(PeriodCommaStyle::WIDELATIN_JAPANESE, PeriodCommaStyle,
-                periodStyle, "anthy-period-widelatin-japanese");
+                periodStyle, "anthy-period-widelatin-japanese")
 
     _ADD_ACTION(SymbolStyle::JAPANESE, SymbolStyle, symbolStyle,
-                "anthy-symbol-japanese");
+                "anthy-symbol-japanese")
     _ADD_ACTION(SymbolStyle::WIDEBRACKET_WIDESLASH, SymbolStyle, symbolStyle,
-                "anthy-symbol-widebracket-wideslash");
+                "anthy-symbol-widebracket-wideslash")
     _ADD_ACTION(SymbolStyle::CORNERBRACKET_MIDDLEDOT, SymbolStyle, symbolStyle,
-                "anthy-symbol-cornerbracket-middledot");
+                "anthy-symbol-cornerbracket-middledot")
     _ADD_ACTION(SymbolStyle::CORNERBRACKET_WIDESLASH, SymbolStyle, symbolStyle,
-                "anthy-symbol-cornerbracket-wideslash");
+                "anthy-symbol-cornerbracket-wideslash")
 
     instance_->inputContextManager().registerProperty("anthyState", &factory_);
 
@@ -433,7 +464,10 @@ std::string AnthyEngine::fullFileName(const std::string &name) {
 
 void AnthyEngine::reloadConfig() {
     fcitx::readAsIni(config_, "conf/anthy.conf");
+    populateConfig();
+}
 
+void AnthyEngine::populateConfig() {
     std::string file;
 
     StyleFile style;
@@ -461,7 +495,7 @@ void AnthyEngine::reloadConfig() {
         FCITX_ANTHY_DEBUG() << "Try loading romaji table: " << filename;
         if (style.load(filename)) {
             customRomajiTableLoaded_ = true;
-            customRomajiTable_ = style.key2kanaTable(section_romaji);
+            customRomajiTable_ = Key2KanaTable(section_romaji, style);
         }
     }
 
@@ -474,7 +508,7 @@ void AnthyEngine::reloadConfig() {
         FCITX_ANTHY_DEBUG() << "Try loading kana table: " << filename;
         if (style.load(filename)) {
             customKanaTableLoaded_ = true;
-            customKanaTable_ = style.key2kanaTable(section_kana);
+            customKanaTable_ = Key2KanaTable(section_kana, style);
         }
     }
 
@@ -486,10 +520,14 @@ void AnthyEngine::reloadConfig() {
         FCITX_ANTHY_DEBUG() << "Try loading nicola table: " << filename;
         if (style.load(filename)) {
             customNicolaTableLoaded_ = true;
-            customNicolaTable_ = style.key2kanaTable(section_nicola);
+            customNicolaTable_ = Key2KanaTable(section_nicola, style);
         }
     }
 
+    populateOptionToState();
+}
+
+void AnthyEngine::populateOptionToState() {
     if (factory_.registered()) {
         instance_->inputContextManager().foreach(
             [this](fcitx::InputContext *ic) {
@@ -497,7 +535,7 @@ void AnthyEngine::reloadConfig() {
                 state->configure();
                 return true;
             });
-    };
+    }
 }
 
 std::string AnthyEngine::subMode(const fcitx::InputMethodEntry &,
@@ -546,12 +584,27 @@ void AnthyEngine::deactivate(const fcitx::InputMethodEntry &,
                              fcitx::InputContextEvent &event) {
     auto anthy = event.inputContext()->propertyFor(&factory_);
     anthy->autoCommit(event);
+    anthy->updateUI();
 }
 
 void AnthyEngine::reset(const fcitx::InputMethodEntry &,
                         fcitx::InputContextEvent &event) {
     auto anthy = event.inputContext()->propertyFor(&factory_);
     anthy->reset();
+    anthy->updateUI();
 }
 
-FCITX_ADDON_FACTORY(AnthyFactory);
+void AnthyEngine::invokeActionImpl(const fcitx::InputMethodEntry &entry,
+                                   fcitx::InvokeActionEvent &event) {
+    const int cursor = event.cursor();
+    if (event.action() != fcitx::InvokeActionEvent::Action::LeftClick ||
+        cursor < 0) {
+        return InputMethodEngineV3::invokeActionImpl(entry, event);
+    }
+    event.filter();
+    auto anthy = event.inputContext()->propertyFor(&factory_);
+    anthy->movePreeditCaret(event.cursor());
+    anthy->updateUI();
+}
+
+FCITX_ADDON_FACTORY(AnthyFactory)
